#ifndef _CEGOTRANSACTIONMANAGER_H_INCLUDED_
#define _CEGOTRANSACTIONMANAGER_H_INCLUDED_
///////////////////////////////////////////////////////////////////////////////
//                                                         
// CegoTransactionManager.h
// ------------------------
// Cego transaction manager class definition
//     
// Design and Implementation by Bjoern Lemke
//
// (C)opyright 2000-2010 Bjoern Lemke
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; see the file COPYING.  If not, write to
// the Free Software Foundation, 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.
//
// INTERFACE MODULE
//
// Class: CegoTransactionManager
// 
// Description: Transaction handling
//
// Status: QG-2.6
//
///////////////////////////////////////////////////////////////////////////////

// base includes
#include <lfc/SetT.h>
#include <lfc/Chain.h>

// cego includes
#include "CegoDefs.h"
#include "CegoTableManager.h"
#include "CegoTableObject.h"
#include "CegoQueryHelper.h"

// forward declaration
class CegoTableManager;

class CegoTransactionManager {
    
 public:

    CegoTransactionManager(CegoTableManager* pTM);
    ~CegoTransactionManager();

    void createSystemSpace(int tabSetId);
       
    void release(int tabSetId);

    void initLock(int tabSetId);
    void reorgSystemSpace(int tabSetId);

    void recordUpdate(int tabSetId, int tid, const CegoDataPointer& dp );
    void commitUpdate(int tabSetId, const Chain& idxName, int tid, bool doAppend);

    void getRBDataPtr(int tabSetId, int tid, int fileId, int pageId, int offset, int& rbfileId, int& rbpageId, int& rboffset);

    void newRBEntry(int tabSetId, int tid, int fileId, int pageId, int offset, const Chain& tableName);
    
    long commitTransaction(int tabSetId, int tid);

    void getTransactionAffectedTables(int tabSetId, int tid, SetT<Chain>& tableList);

    long rollbackTransaction(int tabSetId, int tid);

    void getOpenTid(int tabSetId, SetT<int>& tidList);

    void getTransactionInfo(int tabSetId, int tid, int& undoop, int& updop);
     
 private:
    
    CegoDatabaseManager *_pDBMng;
    CegoTableManager* _pTM;

    ListT<CegoField> _updSchema;
    ListT<CegoField> _rbcatSchema;

    CegoBufferPage _pRBPage[TABMNG_MAXTABSET];
    CegoTableObject _rbe[TABMNG_MAXTABSET];
    CegoDataPointer _rbEntry[TABMNG_MAXTABSET];

    CegoQueryHelper _qh;

    unsigned long _modId;
};

#endif




